package cn.gygxzc.tina.config.shiro.filter

import cn.gygxzc.tina.config.jwt.JwtConfig
import cn.gygxzc.tina.config.jwt.JwtUtils
import cn.gygxzc.tina.config.shiro.cache.JwtRedisCache
import org.apache.commons.lang3.StringUtils
import org.apache.shiro.SecurityUtils
import org.apache.shiro.web.filter.PathMatchingFilter
import org.apache.shiro.web.util.WebUtils
import org.slf4j.LoggerFactory
import javax.servlet.ServletRequest
import javax.servlet.ServletResponse
import javax.servlet.http.HttpServletRequest

/**
 * @program: Zen-Framework
 *
 * @description: ${description}
 *
 * @author: tuonina
 *
 * @create: 2018-09-20 22:20
 * 任然没有想好，这个到底应该这么搞
 **/
class JwtPathMatchingFilter(private val config: JwtConfig,
                            private val cache: JwtRedisCache) : PathMatchingFilter() {
    companion object {
        private const val authenticationName = "Authentication"
        private val logger = LoggerFactory.getLogger(JwtPathMatchingFilter::class.java)
    }

    /**
     * 解析header中包含的Authorization ,jwt 格式的，如果有，绑定对应的授权信息
     */
    override fun onPreHandle(request: ServletRequest, response: ServletResponse, mappedValue: Any?): Boolean {
        val httpRequest = request as HttpServletRequest
        val jwtToken: String? = httpRequest.getHeader(authenticationName)
        val subject = SecurityUtils.getSubject()
        if (!StringUtils.isBlank(jwtToken) && !subject.isAuthenticated) {
            try {
                val jwtPayload = JwtUtils.parseJwt(config.key, jwtToken!!)
                val authToken = cache.get(jwtPayload.username)
                if (authToken != null)
                    subject.login(authToken)
            } catch (e: Exception) {
                logger.info("onPreHandle: {}", e)
            }
        }
        return true
    }
}