package cn.gygxzc.tina.config.shiro.utils

import org.slf4j.LoggerFactory
import java.io.*


/**
 * 序列化工具
 * @author NorthLan
 * @date 2018/4/23
 * @url https://noahlan.com
 */
object SerializeUtils {
    private val logger = LoggerFactory.getLogger(SerializeUtils::class.java)

    /**
     * deserialize
     * @param bytes
     * @return
     */
    fun deserialize(bytes: ByteArray): Any? {
        var result: Any? = null
        if (isEmpty(bytes)) {
            return null
        }

        try {
            val byteStream = ByteArrayInputStream(bytes)
            try {
                val objectInputStream = ObjectInputStream(byteStream)
                try {
                    result = objectInputStream.readObject()
                } catch (ex: ClassNotFoundException) {
                    throw Exception("Failed to deserialize object type", ex)
                }

            } catch (ex: Throwable) {
                throw Exception("Failed to deserialize", ex)
            }

        } catch (e: Exception) {
            logger.error("Failed to deserialize", e)
        }
        return result
    }

    private fun isEmpty(data: ByteArray?): Boolean {
        return data == null || data.isEmpty()
    }

    /**
     * serialize
     * @param object
     * @return
     */
    fun serialize(`object`: Any?): ByteArray? {
        var result: ByteArray? = null
        if (`object` == null) {
            return ByteArray(0)
        }
        try {
            val byteStream = ByteArrayOutputStream(128)
            try {
                if (`object` !is Serializable) {
                    throw IllegalArgumentException(
                            SerializeUtils::class.java.simpleName + " requires a Serializable payload "
                                    + "but received an object of type [" + `object`.javaClass.name + "]")
                }
                val objectOutputStream = ObjectOutputStream(byteStream)
                objectOutputStream.writeObject(`object`)
                objectOutputStream.flush()
                result = byteStream.toByteArray()
            } catch (ex: Throwable) {
                throw Exception("Failed to serialize", ex)
            }
        } catch (ex: Exception) {
            logger.error("Failed to serialize", ex)
        }

        return result
    }
}